﻿var suggestions = {};
var autocompleteWidget;

$(document).ready(function () {

	//Iterate over all the input tags in the document with an attribute data-autocomplete
	$.each($("input[data-autocomplete]"), function (index, item) {
		var entityName = $(item).attr('data-autocomplete');
		fetchSuggestions('/dataset/' + entityName + '.js', entityName);
	});

	//Setup the input tags to display suggestions by binding to their keyup event
	$(document).on("keyup", "input[data-autocomplete]", function (event) {
		var code = event.code || event.which;

		//If the user presses the up arrow or down arrow key, move the selected suggestion as well
		switch (code) {
			case 38: {//Up Arrow key pressed
				var prev = $("a.list-group-item.active").prev();

				if (prev) {
					$("a.list-group-item.active").removeClass("active");
					$(prev).addClass("active");
				}
				break;
			}
			case 40: {//Down Arrow key pressed
				var next = $("a.list-group-item.active").next();
				if (next) {
					$("a.list-group-item.active").removeClass("active");
					$(next).addClass("active");
				}
				break;
			}
			case 13: {//If the user clicks the Enter key
				if (autocompleteWidget && ($(autocompleteWidget).data('bs.modal') || {}).isShown) {
					$(this).val($("a.list-group-item.active").text());
					$(autocompleteWidget).modal('hide');
					break;
				}
			}
			default: {//Display suggestions based on what  the user has typed in so far
				displaySuggestions(getSuggestions($(this).val(), this), this);
			}
		}	
	});

	//Bind to click event of each suggestion displayed and populate the input tag with its value on click
	$(document).on("click", "a.list-group-item", function (e) {
		var code = e.code || e.which;
		var target = e.target;
		if (code !== 1 || !autocompleteWidget || !target) return;
		$(autocompleteWidget["wcg-autocomplete-input"]).val($(target).text());
		$(autocompleteWidget).modal('hide');
	});
});

function getSuggestions(searchTerm, element) {
	var suggestionList = "";
	var firstSuggestion = "<a class='list-group-item active'>";
	var suggestion = "<a class='list-group-item'>";

	if (searchTerm === "") {
		return "";
	}
	var list = $.grep(suggestions[$(element).attr('data-autocomplete')], function (s) {
		return s.toLowerCase().indexOf(searchTerm.toLowerCase()) >= 0;
	});

	$.each(list.slice(0, (list.length > 5 ? 4 : list.length)), function (index, item) {
		if (index === 0) {
			suggestionList = suggestionList + firstSuggestion + item + "</a>";
		} else {
			suggestionList = suggestionList + suggestion + item + "</a>";
		}
	}, this);
	suggestionList = suggestionList;
	return suggestionList;
};

function fetchSuggestions(suggestionsUrl, suggestionsFor) {
	if (suggestions.hasOwnProperty(suggestionsFor)) {
		return;
	}
	$.getJSON(suggestionsUrl, function (response) {
		suggestions[suggestionsFor] = response[suggestionsFor];
	});
}

function displaySuggestions(suggestions, element) {
	if (suggestions.length === 0) {

		if (autocompleteWidget) $(autocompleteWidget).modal('hide');
		return;
	}
	var suggDiv;
	if (autocompleteWidget) {
		suggDiv = $(autocompleteWidget);
	}
	else {
		suggDiv = $("<div id='wcg-modal' class='modal fade' data-backdrop='false'></div>");
	}
	suggDiv.html(suggestions);
	if (!autocompleteWidget) {
		$(document.body).append(suggDiv);
		autocompleteWidget = suggDiv;
	}
	//Place the suggestions list under the correct input tag
	$(autocompleteWidget).css({
		left: $(element).position().left + "px",
		top: $(element).position().top + $(element).height() + 8 + "px"
	});
	autocompleteWidget["wcg-autocomplete-input"] = element;
	$(autocompleteWidget).modal('show');
};